//BEGIN_HSPS_NAMESPACE

#include <string>
#include <numeric>
#include <cmath>
#include <algorithm>

using namespace std;

//const int MAX_B_SIZE = 7;

// ===========================================================================

class Bandit_Planner {
    // static int current_planner_id;
    int planner_id;
    std::vector<float> rewards;
    int trial_limit;
    int total_improved_plan;
    std::vector<double> time_1st_plan;
    std::vector<double> time_last_plan;
    static int current_id;

  public:
    Bandit_Planner():planner_id(current_id++), total_improved_plan(0), trial_limit(1){}
    //    void set_planner_id (int);
    void add_rewards (float);
    void add_all_rewards (vector<float> reward_list){ this->rewards = reward_list;} 
    //    void set_overall_no_of_plays (int);
    std::vector<float> get_rewards () {return rewards;}
    //    Bandit_Size* get_size (){return bsize;}
    int get_planner_id () {return this->planner_id;} 
    int get_total_plays () {return rewards.size();} 
    int get_overall_no_of_plays (Bandit_Planner*, int); 
    //    int get_overall_no_of_plays () {return overall_no_of_plays;}
    //    int g_overall_no_of_plays () {return this->bsize->get_total_plays();}
    float get_average_reward(){return std::accumulate(rewards.begin(),rewards.end(),0.0)/(float)rewards.size();}
    bool has_played () {return this->rewards.empty() ? false : true;}
    
    void increment_improved_plan (){ this->total_improved_plan++;}
    void set_total_improved_plan (int total_improved_plan){ this->total_improved_plan = total_improved_plan;}
    int get_total_improved_plan (){ return this->total_improved_plan;}
    void add_time_1st_plan (double time){ this->time_1st_plan.push_back(time);} 
    void add_time_last_plan (double time){ this->time_last_plan.push_back(time);} 
    void set_time_1st_plan (std::vector<double> time){ this->time_1st_plan = time;} 
    void set_time_last_plan (std::vector<double> time){ this->time_last_plan = time;} 
    std::vector<double> get_time_1st_plan (){ return this->time_1st_plan;}
    std::vector<double> get_time_last_plan (){ return this->time_last_plan;}
    
    void set_trial_limit (int tlimit ){ this->trial_limit = tlimit;}
    void increment_trial_limit(){ this->trial_limit++;}
    int get_trial_limit (){ return this->trial_limit;}
};

int Bandit_Planner::current_id = 0;

// void Bandit_Planner::set_planner_id (int planner_id){
//    this->planner_id = planner_id;
// }

void Bandit_Planner::add_rewards (float reward){
    this->rewards.push_back(reward);
}

int Bandit_Planner::get_overall_no_of_plays (Bandit_Planner *bplanner, int max_bplanner) {
    int temp = 0;
    for(int i = 0; i < max_bplanner; i++){
        if( !bplanner[i].has_played()) continue; // switch_off
        temp += bplanner[i].get_total_plays();
    }
    return temp;
} 

// ===========================================================================
    
class Window {
    int window_id;
    // int cost;
    // int hcost;
    int length_new;
    int cost_improve;
    // vector<bool> planner_used;
    vector<int> trial_count;    
    vector<bool> wpolicies;  
    // double alloted_time;
    double time_1st_plan;
    // double time_last_plan;
    double exhaust_time;
    // int exec_status; // 0:suspended/new; 1:no_solution; 2:reach_max_attempt; 
    bool exhaust; 
    // bool freeze;
    static int current_id;
    string actions_str;
    string actions_new_str;
    // vector<int> blocks;
    string pred_str;
    string succ_str;
    string iatoms_str;
    string gatoms_str;
    // vector<int> new_actions;
    
    vector<bool> frules;
    vector<int> rank; 
    
  public:
    Window():window_id(current_id++),time_1st_plan(-1.0), exhaust_time(-1.0), cost_improve(0), exhaust(false), length_new(-1){
        // planner_used.resize(2);
        // planner_used[0] = false;
        // planner_used[1] = false;
        
        trial_count.resize(3);
        fill(trial_count.begin(), trial_count.end(), 0);
        
        wpolicies.resize(6);
        fill(wpolicies.begin(), wpolicies.end(), false);
        
        frules.resize(20);
        fill(frules.begin(), frules.end(), false);
    }

    // void set_cost(int cost){this->cost = cost;}
    int get_cost(){return this->rank[1];}
    // void set_hcost(int cost){this->hcost = adm_hcost;}
    int get_adm_hcost(){return this->rank[2];}
    void set_new_length(int new_length){this->length_new = new_length;}
    int get_new_length(){return this->length_new;}
    void set_cost_improve(int cost_improve){this->cost_improve = cost_improve;}
    int get_cost_improve(){return this->cost_improve;}
    
    void mark_exhausted(){this->exhaust = true;}
    bool exhausted(){return this->exhaust;}
    
    // void mark_frozen(){this->freeze = true;}
    // bool frozen(){return this->freeze;}
    // void set_planner(int planner_id){this->planner_used[planner_id] = true;}
    // vector<bool> get_planner(){return this->planner_used;}
    
    void increase_trial_count(int pid){this->trial_count[pid]++;}
    int get_trial_count(int pid){return this->trial_count[pid];}
    void reset_trial_count(int pid){this->trial_count[pid] = 0;}
    int get_trial_count_combined(){return this->trial_count[0]+this->trial_count[1];}
    
    void set_time_1st_plan(double time_1st_plan){this->time_1st_plan = time_1st_plan;}
    double get_time_1st_plan(){return this->time_1st_plan;}
    
    // void set_time_last_plan(double time_last_plan){this->time_last_plan = time_last_plan;}
    // double get_time_last_plan(){return this->time_last_plan;}
    
    void set_exhaust_time(double exhaust_time){this->exhaust_time = exhaust_time;}
    double get_exhaust_time(){return this->exhaust_time;}
    // void set_blocks(vector<int> block){this->blocks = block;}
    // vector<int> get_blocks(){return this->blocks;}
    
    void set_actions_str(string actions){this->actions_str = actions;}
    string get_actions_str(){return this->actions_str;}
    
    void set_actions_new_str(string nactions){this->actions_new_str = nactions;}
    string get_actions_new_str(){return this->actions_new_str;}
    
    void set_iatoms_str(string iatoms){this->iatoms_str = iatoms;}
    string get_iatoms_str(){return this->iatoms_str;}
    
    void set_gatoms_str(string gatoms){this->gatoms_str = gatoms;}
    string get_gatoms_str(){return this->gatoms_str;}
    
    void set_pred_str(string pred){this->pred_str = pred;}
    string get_pred_str(){return this->pred_str;}
    void set_succ_str(string succ){this->succ_str = succ;}
    string get_succ_str(){return this->succ_str;}
    // void set_init_atoms(vector<int> init_atoms){this->init_atoms = init_atoms;}
    // vector<int> get_init_atoms(){return this->init_atoms;}
    // void set_goal_atoms(vector<int> goal_atoms){this->goal_atoms = goal_atoms;}
    // vector<int> get_goal_atoms(){return this->goal_atoms;}
    // void set_new_actions(vector<int> new_actions){this->new_actions = new_actions;}
    // vector<int> get_new_actions(){return this->new_actions;}
    // int get_new_action_length(){return this->new_actions.size();}
    
    void set_rank(vector<int> vrank){rank = vrank;}
    vector<int> get_rank(){return rank;}
    
    void set_frule(int rule_id){frules[rule_id] = true;}
    vector<bool> get_frules(){return frules;}
    
    void set_wpolicy(int wpolicy_id){wpolicies[wpolicy_id] = true;}
    vector<bool> get_wpolicies(){return wpolicies;}
    
    // void push_wpolicy(int ws){this->wpolicies.push_back(ws);}
    // vector<int> get_wpolicies(){return this->wpolicies;}
    
    // void push_btype(int bt){this->btype.push_back(bt);}
    // vector<int> get_btype(){return this->btype;}
};

int Window::current_id = 0;
